@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2020 - 2021, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "test.h"
 
@@ -25,12 +27,23 @@
 #include "warnless.h"
 #include "memdebug.h"
 
-static const char *preload_hosts[] = {
-  "1.example.com",
-  "2.example.com",
-  "3.example.com",
-  "4.example.com",
-  NULL /* end of list marker */
+struct entry {
+  const char *name;
+  const char *exp;
+};
+
+static const struct entry preload_hosts[] = {
+#if (SIZEOF_TIME_T < 5)
+  { "1.example.com", "20370320 01:02:03" },
+  { "2.example.com", "20370320 03:02:01" },
+  { "3.example.com", "20370319 01:02:03" },
+#else
+  { "1.example.com", "25250320 01:02:03" },
+  { "2.example.com", "25250320 03:02:01" },
+  { "3.example.com", "25250319 01:02:03" },
+#endif
+  { "4.example.com", "" },
+  { NULL, NULL } /* end of list marker */
 };
 
 struct state {
@@ -42,15 +55,16 @@ static CURLSTScode hstsread(CURL *easy, struct curl_hstsentry *e,
                             void *userp)
 {
   const char *host;
+  const char *expire;
   struct state *s = (struct state *)userp;
   (void)easy;
-  host = preload_hosts[s->index++];
+  host = preload_hosts[s->index].name;
+  expire = preload_hosts[s->index++].exp;
 
   if(host && (strlen(host) < e->namelen)) {
     strcpy(e->name, host);
     e->includeSubDomains = FALSE;
-    strcpy(e->expire, "20370320 01:02:03"); /* curl turns 39 that day
-                                   just before 31-bit time_t overflow */
+    strcpy(e->expire, expire);
     fprintf(stderr, "add '%s'\n", host);
   }
   else
@@ -58,6 +72,16 @@ static CURLSTScode hstsread(CURL *easy, struct curl_hstsentry *e,
   return CURLSTS_OK;
 }
 
+/* verify error from callback */
+static CURLSTScode hstsreadfail(CURL *easy, struct curl_hstsentry *e,
+                                void *userp)
+{
+  (void)easy;
+  (void)e;
+  (void)userp;
+  return CURLSTS_FAIL;
+}
+
 /* check that we get the hosts back in the save */
 static CURLSTScode hstswrite(CURL *easy, struct curl_hstsentry *e,
                              struct curl_index *i, void *userp)
@@ -90,6 +114,19 @@ int test(char *URL)
     curl_easy_setopt(hnd, CURLOPT_HSTS_CTRL, CURLHSTS_ENABLE);
     ret = curl_easy_perform(hnd);
     curl_easy_cleanup(hnd);
+    printf("First request returned %d\n", (int)ret);
+  }
+  hnd = curl_easy_init();
+  if(hnd) {
+    curl_easy_setopt(hnd, CURLOPT_URL, URL);
+    curl_easy_setopt(hnd, CURLOPT_HSTSREADFUNCTION, hstsreadfail);
+    curl_easy_setopt(hnd, CURLOPT_HSTSREADDATA, &st);
+    curl_easy_setopt(hnd, CURLOPT_HSTSWRITEFUNCTION, hstswrite);
+    curl_easy_setopt(hnd, CURLOPT_HSTSWRITEDATA, &st);
+    curl_easy_setopt(hnd, CURLOPT_HSTS_CTRL, CURLHSTS_ENABLE);
+    ret = curl_easy_perform(hnd);
+    curl_easy_cleanup(hnd);
+    printf("Second request returned %d\n", (int)ret);
   }
   curl_global_cleanup();
   return (int)ret;
